﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace ALMatrixLib
{

    public class AlgLibMatrix
    {
        // Trial application to test connection to Excel
        public double SolveQuad(double a, double b, double c)
        {
            double result = 0;
            result = (-b + Math.Sqrt(Math.Pow(b, 2) - 4 * a * c)) / (2 * a);
            return result;
        }
  
         // Inverse of real matrix    
        public double [,]    alrminverse(double[,] ddata, out int info)
        {
            alglib.matinvreport rep;
            alglib.rmatrixinverse(ref ddata, out info, out rep);
            return ddata ;
        }

        // Inverse of real matrix using LU decomposition
        public double[,] alrmluinverse(double[,] ddata, int m, out int info)
        {
            int[] pivots;
            alglib.rmatrixlu(ref ddata, m, m, out pivots);
            alglib.matinvreport rep;
            alglib.rmatrixluinverse(ref ddata, pivots, out info, out rep);
            return ddata;
        }

        // Inverse of a triangular matrix    
        public double[,] altriminverse(double[,] ddata, int n, bool isupper, bool isunit, out int info)
        {
            alglib.matinvreport rep;
            alglib.rmatrixtrinverse(ref ddata, n, isupper, isunit, out info, out rep);
            return ddata;
        }

        // Inverse of complex matrix    
        public double[,] alcminverse(double[,] ddata, int n, out int info)
        {
            alglib.complex[,] cdata;
            alglib.matinvreport rep;
            cdata = doublepaira2complexa2(ddata, n, n*2);
            alglib.cmatrixinverse(ref cdata, n, out info, out rep);
            ddata = complexa22doublepaira(cdata, n, n*2);
            return ddata;
        }

        // Inverse of symmetric positive definite matrix    
        public double[,]   alspdminverse(double[,] ddata, int n, bool isupper, out int info)
        {
            alglib.matinvreport rep;
            alglib.spdmatrixinverse(ref ddata, n, isupper,  out info, out rep);
            return ddata ;
        }

        // Determinant of real matrix    
        public double alrmdet(double[,] ddata, int n)
        {
            double  res;
            res = alglib.rmatrixdet(ddata, n);
            return res;
        }

        // Determinant of real matrix using LU decomposition    
        public double alrmludet(double[,] ddata, int n)
        {
            int[] pivots;
            alglib.rmatrixlu(ref ddata, n, n, out pivots);
            double res;
            res = alglib.rmatrixludet(ddata, pivots, n);
            return res;
        }

        // Determinant of complex matrix    
        public double[] alcmdet(double[,] ddata, int n)
        {
            double[] res;
            alglib.complex resc;
            alglib.complex[,] cdata;
            cdata = doublepaira2complexa2(ddata, n, n*2);
            resc = alglib.cmatrixdet(cdata, n);
            res = complex2doublea(resc);
            return res;
        }

        // Transpose of a real matrix
        public double[,] alrmtrans(double[,] ddata, int m, int n)
        {
            double[,] res = new double[n, m];
            alglib.rmatrixtranspose(m, n, ddata, 0, 0, ref res, 0, 0);
            return res;
        }

        // Transpose of complex matrix    
        public double[,] alcmtrans(double[,] ddata, int m, int n)
        {
            alglib.complex[,] cdata;
            alglib.complex[,] ctransmat = new alglib.complex[n / 2, m];
            double[,] rdata = new double[m, n];
            cdata = doublepaira2complexa2(ddata, m, n);
            alglib.cmatrixtranspose(m, n/2, cdata, 0, 0, ref ctransmat, 0, 0);
            rdata = complexa22doublepaira(ctransmat, m, n);
            return rdata;
        }

        // Multiplication and addition of real matrices
        public double[,] alrmmult(int m, int n, int k, double alpha, double[,] ddata1, 
            int ia, int ja, int optypea, double[,] ddata2,
            int ib, int jb, int optypeb, double beta, double[,] ddata3, int ic, int jc)
        {
            alglib.rmatrixgemm(m, n, k, alpha, ddata1, ia, ja, optypea, ddata2, ib, jb, optypeb, beta, ref ddata3, ic, jc);
            return ddata3 ;
        }

        // Multiplication and addition of complex matrices
        public double[,] alcmmult(int m, int n, int k, double alpha, double[,] ddata1,
            int ia, int ja, int optypea, double[,] ddata2,
            int ib, int jb, int optypeb, double beta, double[,] ddata3, int ic, int jc)
        {
            alglib.complex[,] cdata1;
            alglib.complex[,] cdata2;
            alglib.complex[,] cdata3;
            cdata1 = doublepaira2complexa2(ddata1, m, k );
            cdata2 = doublepaira2complexa2(ddata2, k/2 , n );
            cdata3 = doublepaira2complexa2(ddata3, m , n);
            alglib.cmatrixgemm(m, n/2, k/2, alpha, cdata1, ia, ja, optypea, cdata2, ib, jb, optypeb, beta, ref cdata3, ic, jc);
            ddata3 = complexa22doublepaira(cdata3, m, n);
            return ddata3;
        }

        // Addition of real matices
        public double[,] alrmadd(double[,] a, double facta, double[,] b, double factb, int m, int n)
        {
            // alglib.complex[,] c = new alglib.complex[m, n / 2];

            int i, j;
            for (i = 0; i < m; i++)
            {
                for (j = 0; j < n / 2; j++)
                {
                    if (factb == 0.0)
                    {
                        a[i, j] = a[i, j] * facta;
                    }
                    else
                    {
                        a[i, j] = a[i, j] * facta + b[i, j] * factb;
                    }
                }
            }
            return a ;
        }

        // Solve system of real linear equations
        public double[] alrmsolve(double[,] ddata, int n, double[] ddata2, out int info)
        {
           alglib.densesolverreport rep;
            double[] x;
            alglib.rmatrixsolve(ddata, n, ddata2, out info, out rep, out x);
            return x;
        }

        // Solve of system real linear equations using LU decomposition
        public double[] alrmlusolve(double[,] ddata, int n, double[] ddata2, out int info)
        {
            int[] pivots;
            alglib.rmatrixlu(ref ddata, n, n, out pivots);
            alglib.densesolverreport rep;
            double[] x;
            alglib.rmatrixlusolve(ddata, pivots,n, ddata2, out info, out rep, out x);
            return x;
        }

        // Eigen values and vectors of a real matrix
        public bool alrmatevd(double[,] ddata, int n, int vneeded, out double[] wr, out double[] wi, out double[,] vl, out double[,] vr)
        {
            bool res;
           res = alglib.rmatrixevd(ddata, n, vneeded, out wr, out wi, out vl, out vr);
            return res;
        }

        // Eigen values and vectors of a real symmetric matrix
        public bool alrsmatevd(double[,] ddata, int n, int zneeded, bool isupper, out double[] w, out double[,] z)
        {
            bool res;
            res = alglib.smatrixevd(ddata, n, zneeded, isupper, out w, out z);
            return res;
        }

        // Eigen values and vectors of a real symmetric matrix using bisection and inverse iteration
        public bool alrsmatevdi(double[,] ddata, int n, int zneeded, bool isupper, int i1, int i2, out double[] w, out double[,] z)
        {
            bool res;
            res = alglib.smatrixevdi(ddata, n, zneeded, isupper, i1, i2, out w, out z);
            return res;
        }

        /* Eigen values and vectors solving the following generalized symmetric positive-definite
            eigenproblem:
                A*x = lambda*B*x (1) or
                A*B*x = lambda*x (2) or
                B*A*x = lambda*x (3).
            where A is a symmetric matrix, B - symmetric positive-definite matrix.
            The problem is solved by reducing it to an ordinary  symmetric  eigenvalue
            problem. */

        public bool alsmatgevd(double[,] ddata1, int n, bool isuppera, double[,] ddata2, bool isupperb, int zneeded, int problemtype,
            out double[] d, out double[,] z)
        {
            bool res;
            res = alglib.smatrixgevd(ddata1, n, isuppera, ddata2, isupperb, zneeded, problemtype, out d, out z);
            return res;
        }

        // Solve system of real symmetric linear equations with sparse solver
        public double[] alsparsermsolve(int[,] idata, double[] ddata, int n, int sumn, double[] ddata2, out int info)
        {
            int i;
             double[] x;
            alglib.sparsematrix a;
            alglib.sparsecreate(n, n, out a);
            for (i = 0; i <= sumn-1; i++)
            {
                alglib.sparseset(a, idata[i,0], idata[i, 1], ddata[i]);
            }

            alglib.sparseconverttocrs(a);
            alglib.lincgstate s;
            alglib.lincgreport rep;
            
            alglib.lincgcreate(n, out s);
            alglib.lincgsolvesparse(s, a, true, ddata2);
            alglib.lincgresults(s, out x, out rep);
            info =  rep.terminationtype;
             //alglib.sparseget(a, 0, 0);
            return x  ;
        }
        
        // Two doubles to complex value
        public alglib.complex doublepair2complex(double a, double b)
        {
            alglib.complex c;
            c.x = a;
            c.y = b;
            return c;
        }

        // Complex value to 2 member array of doubles
        public double[] complex2doublea(alglib.complex a)
        {
            double[]b = new double[2];
            b[0] = a.x ;
            b[1] = a.y ;
            return b ;
        }

        // nx2 array of doubles to 1D array of n complex values
        public alglib.complex[] doublepaira2complexa1(double[,] a, int n)
        {
            alglib.complex[] c = new alglib.complex[n];
            
            int i;
            for (i = 0; i < n; i++)
                c[i] = doublepair2complex(a[i, 0], a[i, 1]);
            return c;
        }

        // 1D array of n complex values to nx2 array of doubles
        public double[,] complexa12doublepaira(alglib.complex[] a, int n)
        {
            double[,] b = new double[n, 2];

            int i;
            for (i = 0; i < n; i++)
            {
                b[i, 0] = a[i].x;
                b[i, 1] = a[i].y;
            }
            return b ;
        }

        // mxn array of doubles to mxn/2 array of complex
        public alglib.complex[,] doublepaira2complexa2(double[,] a, int m, int n)
        {
            alglib.complex[,] c = new alglib.complex[m, n/2];

            int i, j;
            for (i = 0; i < m ; i++)
            {
                for (j = 0; j < n/2; j++)
                {
                    c[i, j] = doublepair2complex(a[i, j * 2], a[i, j * 2 + 1]);
                }
            }
            return c;
        }


        // mxn/2 array of complex to mxn array of doubles
        public double[,] complexa22doublepaira(alglib.complex[,] a, int m, int n)
        {
            double[,] b = new Double [m, n];

            int i, j;
            for (i = 0; i < m ; i++)
            {
                for (j = 0; j < n/2; j++)
                {
                    b[i, j * 2] = a[i,j].x;
                    b[i, j * 2+1] = a[i, j].y;
                }
            }
            return b ;

        }

        // Check real to complex and complex to real functions
        public double[] checkcomplex1(double a, double b )
        {
            alglib.complex c;
            double[] d = new double[2];
            c = doublepair2complex(a, b );
            d = complex2doublea(c);
            return d ;
        }

        public double[,] checkcomplex2(double[,] a, int m, int n)
        {
            alglib.complex[] b;
            double[,] c;
            b = doublepaira2complexa1(a, m);
            c = complexa12doublepaira(b, m);
            return c;
        }

        public double[,] checkcomplex3(double[,] a, int m, int n)
        {
            alglib.complex[,] b;
            double[,] c;
            b = doublepaira2complexa2(a, m, n );
            c = complexa22doublepaira(b, m, n);
            return c;
        }
    }
}
